"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTokenToInactiveVaultDirect = exports.addTokenToInactiveVault = exports.SafetyDepositSetup = void 0;
const assert_1 = require("assert");
const web3_js_1 = require("@safecoin/web3.js");
const consts_1 = require("../common/consts");
const helpers_1 = require("../common/helpers");
const helpers_mint_1 = require("../common/helpers.mint");
const generated_1 = require("../generated");
class SafetyDepositSetup {
    constructor(vault, tokenAccount, tokenMint, store, safetyDeposit, transferAuthority, transferAuthorityPair, mintAmount, instructions, signers, tokenMintPair) {
        this.vault = vault;
        this.tokenAccount = tokenAccount;
        this.tokenMint = tokenMint;
        this.store = store;
        this.safetyDeposit = safetyDeposit;
        this.transferAuthority = transferAuthority;
        this.transferAuthorityPair = transferAuthorityPair;
        this.mintAmount = mintAmount;
        this.instructions = instructions;
        this.signers = signers;
        this.tokenMintPair = tokenMintPair;
    }
    static async create(connection, args) {
        const { payer, vault, associateTokenAccount = true } = args;
        const instructions = [];
        const signers = [];
        let tokenMint;
        let tokenMintPair;
        const mintRentExempt = await (0, helpers_1.getMintRentExempt)(connection);
        if (args.tokenMint != null) {
            tokenMint = args.tokenMint;
            const info = await connection.getAccountInfo(tokenMint);
            (0, assert_1.strict)(info != null, 'provided mint needs to exist');
            (0, assert_1.strict)(info.lamports >= mintRentExempt, 'provided mint needs to be rent exempt');
            const mint = await (0, helpers_mint_1.getMint)(connection, tokenMint);
            assert_1.strict.equal(mint.decimals, 0, 'provided mint should have 0 decimals');
        }
        else {
            const [createMintIxs, createMintSigners, { mintAccount, mintAccountPair }] = (0, helpers_1.createMint)(payer, mintRentExempt, 0, payer, payer);
            instructions.push(...createMintIxs);
            signers.push(...createMintSigners);
            tokenMint = mintAccount;
            tokenMintPair = mintAccountPair;
        }
        let tokenAccount;
        if (args.tokenAccount != null) {
            tokenAccount = args.tokenAccount;
        }
        else {
            (0, assert_1.strict)(associateTokenAccount, 'only allowing associated token accounts for now');
            const [createAtaIx, associatedTokenAccount] = await (0, helpers_1.createAssociatedTokenAccount)({
                payer,
                tokenOwner: payer,
                tokenMint,
            });
            tokenAccount = associatedTokenAccount;
            instructions.push(createAtaIx);
        }
        const addTokensIx = (0, helpers_1.mintTokens)(tokenMint, tokenAccount, payer, args.mintAmount);
        instructions.push(addTokensIx);
        const [createStoreIxs, createStoreSigners, { tokenAccount: storeAccount }] = await (0, helpers_1.createVaultOwnedTokenAccount)(connection, payer, vault, tokenMint);
        instructions.push(...createStoreIxs);
        signers.push(...createStoreSigners);
        const safetyDepositAccount = await getSafetyDepositAccount(vault, tokenMint);
        const [approveTransferIx, transferAuthorityPair] = (0, helpers_1.approveTokenTransfer)({
            owner: payer,
            sourceAccount: tokenAccount,
            amount: args.mintAmount,
        });
        instructions.push(approveTransferIx);
        return new SafetyDepositSetup(vault, tokenAccount, tokenMint, storeAccount, safetyDepositAccount, transferAuthorityPair.publicKey, transferAuthorityPair, args.mintAmount, instructions, signers, tokenMintPair);
    }
}
exports.SafetyDepositSetup = SafetyDepositSetup;
async function addTokenToInactiveVault(safetyDepositSetup, ixAccounts) {
    const { vault, safetyDeposit, transferAuthority, store, tokenAccount, mintAmount } = safetyDepositSetup;
    const accounts = {
        safetyDepositAccount: safetyDeposit,
        tokenAccount,
        store,
        transferAuthority,
        vault,
        payer: ixAccounts.payer,
        vaultAuthority: ixAccounts.vaultAuthority,
    };
    const instructionAccounts = {
        ...accounts,
        systemAccount: web3_js_1.SystemProgram.programId,
    };
    return (0, generated_1.createAddTokenToInactiveVaultInstruction)(instructionAccounts, {
        amountArgs: { amount: mintAmount },
    });
}
exports.addTokenToInactiveVault = addTokenToInactiveVault;
async function addTokenToInactiveVaultDirect(amountArgs, accounts) {
    const instructionAccounts = {
        ...accounts,
        systemAccount: web3_js_1.SystemProgram.programId,
    };
    return (0, generated_1.createAddTokenToInactiveVaultInstruction)(instructionAccounts, { amountArgs });
}
exports.addTokenToInactiveVaultDirect = addTokenToInactiveVaultDirect;
async function getSafetyDepositAccount(vault, tokenMint) {
    const [pda] = await web3_js_1.PublicKey.findProgramAddress([Buffer.from(consts_1.VAULT_PREFIX), vault.toBuffer(), tokenMint.toBuffer()], consts_1.VAULT_PROGRAM_ID);
    return pda;
}
//# sourceMappingURL=add-token-to-inactive-vault.js.map