"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.combineVault = exports.CombineVaultSetup = void 0;
const web3_js_1 = require("@safecoin/web3.js");
const helpers_1 = require("../common/helpers");
const generated_1 = require("../generated");
const assert_1 = require("assert");
const safe_token_1 = require("@safecoin/safe-token");
class CombineVaultSetup {
    constructor(connection, vault, vaultAuthority, newVaultAuthority, fractionMint, fractionTreasury, redeemTreasury, priceMint, externalPricing, fractionBurnAuthority) {
        this.connection = connection;
        this.vault = vault;
        this.vaultAuthority = vaultAuthority;
        this.newVaultAuthority = newVaultAuthority;
        this.fractionMint = fractionMint;
        this.fractionTreasury = fractionTreasury;
        this.redeemTreasury = redeemTreasury;
        this.priceMint = priceMint;
        this.externalPricing = externalPricing;
        this.fractionBurnAuthority = fractionBurnAuthority;
        this.instructions = [];
        this.signers = [];
    }
    static async create(connection, accounts) {
        const { newVaultAuthority = accounts.vaultAuthority, vault } = accounts;
        const fractionBurnAuthority = await (0, helpers_1.pdaForVault)(vault);
        return new CombineVaultSetup(connection, accounts.vault, accounts.vaultAuthority, newVaultAuthority, accounts.fractionMint, accounts.fractionTreasury, accounts.redeemTreasury, accounts.priceMint, accounts.externalPricing, fractionBurnAuthority);
    }
    async createOutstandingShares(payer) {
        (0, assert_1.strict)(!this.hasOutstandingShares(), 'cannot provide/create outstanding shares twice');
        const rentExempt = await (0, helpers_1.getTokenRentExempt)(this.connection);
        const [instructions, signers, { tokenAccount, tokenAccountPair }] = (0, helpers_1.createTokenAccount)(payer, rentExempt, this.fractionMint, payer);
        this.instructions.push(...instructions);
        this.signers.push(...signers);
        this.yourOutstandingShares = tokenAccount;
        this.yourOutstandingSharesPair = tokenAccountPair;
        return this;
    }
    async createPayment(payer) {
        (0, assert_1.strict)(!this.hasPayment(), 'cannot provide/create payment twice');
        const rentExempt = await (0, helpers_1.getTokenRentExempt)(this.connection);
        const [instructions, signers, { tokenAccount, tokenAccountPair }] = (0, helpers_1.createTokenAccount)(payer, rentExempt, this.priceMint, payer);
        this.instructions.push(...instructions);
        this.signers.push(...signers);
        this.yourPayment = tokenAccount;
        this.yourPaymentPair = tokenAccountPair;
        return this;
    }
    approveTransfers(payer) {
        (0, assert_1.strict)(this.hasOutstandingShares(), 'need to provide or create outstandingShares first');
        (0, assert_1.strict)(this.hasPayment(), 'need to provide or create payment first');
        (0, assert_1.strict)(!this.hasTransferAuthority(), 'cannot approve twice');
        const transferAuthorityPair = web3_js_1.Keypair.generate();
        this.transferAuthority = transferAuthorityPair.publicKey;
        this.transferAuthorityPair = transferAuthorityPair;
        const instructions = [
            (0, safe_token_1.createApproveInstruction)(this.yourOutstandingShares, this.transferAuthority, payer, 0),
            (0, safe_token_1.createApproveInstruction)(this.yourPayment, this.transferAuthority, payer, 0),
        ];
        this.instructions.push(...instructions);
        return this;
    }
    hasOutstandingShares() {
        return this.yourOutstandingShares != null;
    }
    hasPayment() {
        return this.yourPayment != null;
    }
    hasTransferAuthority() {
        return this.transferAuthority != null && this.transferAuthorityPair != null;
    }
    assertComplete() {
        (0, assert_1.strict)(this.hasOutstandingShares(), 'need to provide or create outstandingShares');
        (0, assert_1.strict)(this.hasPayment(), 'need to provide or create payment');
        (0, assert_1.strict)(this.hasTransferAuthority(), 'need to approve transfer or add transferAuthority');
    }
    get accounts() {
        this.assertComplete();
        return {
            vault: this.vault,
            vaultAuthority: this.vaultAuthority,
            newVaultAuthority: this.newVaultAuthority,
            fractionMint: this.fractionMint,
            fractionTreasury: this.fractionTreasury,
            redeemTreasury: this.redeemTreasury,
            yourOutstandingShares: this.yourOutstandingShares,
            yourPayment: this.yourPayment,
            transferAuthority: this.transferAuthority,
            fractionBurnAuthority: this.fractionBurnAuthority,
            externalPricing: this.externalPricing,
        };
    }
}
exports.CombineVaultSetup = CombineVaultSetup;
async function combineVault(combineSetup) {
    return (0, generated_1.createCombineVaultInstruction)(combineSetup.accounts);
}
exports.combineVault = combineVault;
//# sourceMappingURL=combine-vault.js.map