"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initVault = exports.VaultSetup = void 0;
const safe_token_1 = require("@safecoin/safe-token");
const web3_js_1 = require("@safecoin/web3.js");
const consts_1 = require("../common/consts");
const helpers_1 = require("../common/helpers");
const generated_1 = require("../generated");
const assert_1 = require("assert");
class VaultSetup {
    constructor(connection, vaultPda, vaultPair, vaultAuthority, priceMint, externalPriceAccount) {
        this.connection = connection;
        this.vaultPda = vaultPda;
        this.vaultPair = vaultPair;
        this.vaultAuthority = vaultAuthority;
        this.priceMint = priceMint;
        this.externalPriceAccount = externalPriceAccount;
        this.instructions = [];
        this.signers = [];
    }
    static async create(connection, { vaultAuthority, priceMint, externalPriceAccount, }) {
        const { vaultPair, vaultPda } = await vaultAccountPDA();
        return new VaultSetup(connection, vaultPda, vaultPair, vaultAuthority, priceMint, externalPriceAccount);
    }
    async supplyFractionMint(fractionMint, currentMintAuthority) {
        const transferAuthIx = (0, safe_token_1.createSetAuthorityInstruction)(fractionMint, currentMintAuthority.publicKey, safe_token_1.AuthorityType.MintTokens, this.vaultPda);
        this.instructions.push(transferAuthIx);
        this.signers.push(currentMintAuthority);
        this.fractionMint = fractionMint;
        this.fractionMintAuthority = this.vaultPda;
        return this;
    }
    async createFracionMint(payer) {
        const mintRentExempt = await this.connection.getMinimumBalanceForRentExemption(safe_token_1.MintLayout.span);
        const [fractionMintIxs, fractionMintSigners, { mintAccount }] = (0, helpers_1.createMint)(payer, mintRentExempt, 0, this.vaultPda, this.vaultPda);
        this.instructions.push(...fractionMintIxs);
        this.signers.push(...fractionMintSigners);
        this.fractionMint = mintAccount;
        this.fractionMintAuthority = this.vaultPda;
        return this;
    }
    async createFractionTreasury(payer) {
        (0, assert_1.strict)(this.hasFractionMint(), 'supply or create fraction mint first');
        const tokenAccountRentExempt = await this.connection.getMinimumBalanceForRentExemption(safe_token_1.AccountLayout.span);
        const [fractionTreasuryIxs, fractionTreasurySigners, { tokenAccount }] = (0, helpers_1.createTokenAccount)(payer, tokenAccountRentExempt, this.fractionMint, this.vaultPda);
        this.instructions.push(...fractionTreasuryIxs);
        this.signers.push(...fractionTreasurySigners);
        this.fractionTreasury = tokenAccount;
        return this;
    }
    async createRedeemnTreasury(payer) {
        const tokenAccountRentExempt = await this.connection.getMinimumBalanceForRentExemption(safe_token_1.AccountLayout.span);
        const [redeemTreasuryIxs, redeemTreasurySigners, { tokenAccount }] = (0, helpers_1.createTokenAccount)(payer, tokenAccountRentExempt, this.priceMint, this.vaultPda);
        this.instructions.push(...redeemTreasuryIxs);
        this.signers.push(...redeemTreasurySigners);
        this.redeemTreasury = tokenAccount;
        return this;
    }
    async createVault(payer) {
        const vaultRentExempt = await generated_1.Vault.getMinimumBalanceForRentExemption(this.connection);
        const uninitializedVaultIx = web3_js_1.SystemProgram.createAccount({
            fromPubkey: payer,
            newAccountPubkey: this.vaultPair.publicKey,
            lamports: vaultRentExempt,
            space: generated_1.Vault.byteSize,
            programId: consts_1.VAULT_PROGRAM_ID,
        });
        this.instructions.push(uninitializedVaultIx);
        this.signers.push(this.vaultPair);
        return this;
    }
    getAccounts() {
        this.assertComplete();
        return {
            fractionMint: this.fractionMint,
            fractionTreasury: this.fractionTreasury,
            redeemTreasury: this.redeemTreasury,
            vault: this.vaultPair.publicKey,
            authority: this.vaultAuthority,
            pricingLookupAddress: this.externalPriceAccount,
        };
    }
    hasFractionMint() {
        return this.fractionMint != null && this.fractionMintAuthority != null;
    }
    hasFractionTreasury() {
        return this.fractionTreasury != null;
    }
    hasRedeemTreasury() {
        return this.redeemTreasury != null;
    }
    assertComplete() {
        (0, assert_1.strict)(this.hasFractionMint(), 'need to supply or create fraction mint');
        (0, assert_1.strict)(this.hasFractionTreasury(), 'need to create fraction treasury');
        (0, assert_1.strict)(this.hasRedeemTreasury(), 'need to create redeem treasury');
    }
}
exports.VaultSetup = VaultSetup;
function initVault(vaultSetup, allowFurtherShareCreation) {
    const accounts = vaultSetup.getAccounts();
    const initVaultArgs = {
        initVaultArgs: { allowFurtherShareCreation },
    };
    return (0, generated_1.createInitVaultInstruction)(accounts, initVaultArgs);
}
exports.initVault = initVault;
async function vaultAccountPDA() {
    const vaultPair = web3_js_1.Keypair.generate();
    const vaultPda = await (0, helpers_1.pdaForVault)(vaultPair.publicKey);
    return { vaultPair, vaultPda };
}
//# sourceMappingURL=init-vault.js.map